#include "testApp.h"


//--------------------------------------------------------------
void testApp::setup(){	
	
	ofSetVerticalSync(true);
	ofSetFrameRate(60);
	ofSetRectMode(OF_RECTMODE_CENTER );
	
	
	for (int i = 0; i < 100; i++){
		particle myParticle;
		myParticle.setInitialCondition(ofRandom(0,ofGetWidth()),ofRandom(0,ofGetHeight()),0,0);
		particles.push_back(myParticle);
	}
	
	
	VF.setupField(60,40,ofGetWidth(), ofGetHeight());
//	VF.randomizeField(2.0);
	
	drawingStyle = 0;
	bFade = false;
}

//--------------------------------------------------------------
void testApp::update(){

	// on every frame 
	// we reset the forces
	// add in any forces on the particle
	// perfom damping and
	// then update
	
	
	for (int i = 0; i < particles.size(); i++){
		particles[i].resetForce();
		
		for (int j = 0; j < i; j++){
			particles[i].addRepulsionForce(particles[j], 10, 5.0);
			particles[i].addRepulsionForce(particles[j], 50, 0.8);
			particles[i].addAttractionForce(particles[j], 1200, 0.001);
			if (abs(particles[i].pos.x - particles[j].pos.x) < 100 && abs(particles[i].pos.y - particles[j].pos.y) < 100) {
				particles[i].addRepulsionForce(particles[j], 90, 0.0001);
			}
		}
	
		// get the force from the vector field: 
		ofxVec2f frc;
		
		frc = VF.getForceFromPos(particles[i].pos.x, particles[i].pos.y);
		
		particles[i].addForce(frc.x, frc.y);
		
		particles[i].addDampingForce();
		
		particles[i].update();
		
		particles[i].bounceOffWalls();
		
	}

	
	if (bFade == true) VF.fadeField(0.99f);

}

//--------------------------------------------------------------
void testApp::draw(){
	ofBackground(0, 0, 0);
	ofEnableAlphaBlending();
//	ofSetColor(0,130,130, 200);
	VF.draw();
	
	ofSetColor(255,255,255,100);
	
	for (int i = 0; i < particles.size(); i++){
		particles[i].draw();
		for (int j = 0; j < i; j++){
			if (abs(particles[i].pos.x - particles[j].pos.x) < 100 && abs(particles[i].pos.y - particles[j].pos.y) < 100) {
				ofLine(particles[i].pos.x, particles[i].pos.y, particles[j].pos.x, particles[j].pos.y);
			}
		}
	}
	
	ofSetColor(0,130,130, 200);
	ofRect(30,30,300,75);
	ofSetColor(255,255,255, 230);
	ofDrawBitmapString("space to clear\nchange drawing mode 'a'\ntoggle fade 'f'", 50, 50);
	
	
	ofSetColor(255,255,130, 230);
	switch (drawingStyle){
		case 0: ofDrawBitmapString("drawing mode: inward", 50, 90);
			break;
		case 1: ofDrawBitmapString("drawing mode: outward", 50, 90);
			break;
		case 2: ofDrawBitmapString("drawing mode: clockwise", 50, 90);
			break;
		case 3: ofDrawBitmapString("drawing mode: counter-clockwise", 50, 90); 
			break;
	}	
}

//--------------------------------------------------------------
void testApp::keyPressed  (int key){ 
	
	if (key == ' '){
		VF.clear();
	} else if (key == 'a'){
		drawingStyle ++;
		drawingStyle %= 4;
	} else if (key == 'f'){
		bFade = !bFade;
	}
}

//--------------------------------------------------------------
void testApp::keyReleased  (int key){ 
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	if (button == 0) {
		switch (drawingStyle){
			case 0: VF.addInwardCircle((float)x, (float)y, 200, 0.3f);
				break;
			case 1: VF.addOutwardCircle((float)x, (float)y, 100, 0.3f);
				break;
			case 2: VF.addClockwiseCircle((float)x, (float)y, 100, 0.3f);
				break;
			case 3: VF.addCounterClockwiseCircle((float)x, (float)y, 100, 0.3f);
				break;
		}		
	} else {
		
		particles.erase(particles.begin());
		particle myParticle;
		myParticle.setInitialCondition(x,y,0,0);
		particles.push_back(myParticle);
		
		
	}
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
}

//--------------------------------------------------------------
void testApp::mouseReleased(){
}
